#include <linux/module.h>

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/in.h>
#include <linux/malloc.h>
#include <linux/string.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <linux/errno.h>
#include <linux/init.h>

#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>

#ifdef MODULE

struct cloud_data
{
	struct net_device_stats stats;
};

static struct net_device_stats *
cloud_get_stats(struct device *dev)
{
	return &((struct cloud_data *) dev->priv)->stats;
}

static int
cloud_open(struct device *dev)
{
	MOD_INC_USE_COUNT;
	return 0;
}

static int
cloud_close(struct device *dev)
{
	MOD_DEC_USE_COUNT;
	return 0;
}

static int
cloud_xmit(struct sk_buff *skb, struct device *dev)
{
	struct cloud_data *d;
	struct net_device_stats *stats;

	d = (struct cloud_data *) dev->priv;
	stats = &d->stats;

	/*
	 *	Optimise so buffers with skb->free=1 are not copied but
	 *	instead are lobbed from tx queue to rx queue 
	 *      (see loopback.c - I don't understand this code.)
	 */
	if(atomic_read(&skb->users) != 1)
	{
	  	struct sk_buff *skb2=skb;
	  	skb=skb_clone(skb, GFP_ATOMIC);		/* Clone the buffer */
	  	if(skb==NULL) {
			kfree_skb(skb2);
			return 0;
		}
	  	kfree_skb(skb2);
	}
	else
		skb_orphan(skb);

	if (stats->tx_packets > 50)
		return 0;
	skb->dev = dev;
	netif_rx(skb);
	stats->tx_packets++;
	stats->tx_bytes += skb->len;
	stats->rx_packets++;
	stats->rx_bytes += skb->len;
	return 0;
}

static int
cloud_init(struct device *dev)
{
	dev->open		= cloud_open;
	dev->stop               = cloud_close;
	dev->hard_start_xmit	= cloud_xmit;
	dev->get_stats          = cloud_get_stats;
	ether_setup(dev);
	dev->priv = kmalloc(sizeof(struct cloud_data), GFP_KERNEL);
	if (dev->priv == NULL)
			return -ENOMEM;
	memset(dev->priv, 0, sizeof(struct cloud_data));
	return(0);
}

static struct device cloud0_dev = {
	"cloud0", 
	0, 0,
	0, 0,
	0, 0,
	0, 0, 0,     /* flags */
	NULL,
	cloud_init
};

static struct device cloud1_dev = {
	"cloud1", 
	0, 0,
	0, 0,
	0, 0,
	0, 0, 0,     /* flags */
	NULL,
	cloud_init
};

int
init_module(void)
{
	int rv;
	rv = register_netdev(&cloud0_dev);
	rv = register_netdev(&cloud1_dev);
	if (!rv)
		printk("cloud net module loaded\n");
	return 0;
}

void
cleanup_module(void)
{
	unregister_netdev(&cloud0_dev);
	if (cloud0_dev.priv)
		kfree_s(cloud0_dev.priv, sizeof(struct cloud_data));
	unregister_netdev(&cloud1_dev);
	if (cloud1_dev.priv)
		kfree_s(cloud1_dev.priv, sizeof(struct cloud_data));
}

#endif /* MODULE */

/*
 * Local variables: ***
 * compile-command: "gcc -D__KERNEL__ -DMODULE -Wall -O2 -c cloud.c" ***
 * End: ***
 */
