/* 
 *  rocks/insert_code.h
 *
 *  Inserts code into the child process to load the rocks library.
 *  Uses file entry.bin and preload.bin generated by NASM.
 *
 *  Copyright (C) 2001 Victor Zandy, Greg Quinn
 *  See COPYING for distribution terms.
 */

#ifndef _INSERT_CODE_H_
#define _INSERT_CODE_H_

#define ENTRY_BIN_NAME "entry.bin"
#define ENTRY_BIN_SIZE 7				/* size of entry binary file */

#define PRELOAD_BIN_NAME "preload.bin"
#define PRELOAD_BIN_SIZE 76				/* size of preload binary file */

/* jump target for new entry code relative to preload_address */
#define PRELOAD_START_OFFSET 0x11

/* offsets for patches needed for entry code */
#define EP_PRELOAD_ADDRESS 0x1		/* address of alloc'ed new code area */

/* offsets needed to compute addresses */
#define P_ROCKSDLL 0x0					/* offset for rocks DLL string */

/* offsets for patches needed for preload code */
#define PP_OLD_ENTRY_BUF 0xA			/* old entry code */
#define PP_LOAD_LIBRARY 0x18			/* LoadLibrary */
#define PP_ENTRY_POINT_1 0x31			/* address of entry point */
#define PP_GET_CURRENT_PROCESS 0x37		/* GetCurrentProcess */
#define PP_WRITE_PROCESS_MEMORY 0x3F	/* WriteProcessMemory */
#define PP_ENTRY_POINT_2 0x46			/* address of entry point */
#define PP_DLLSTRING_ADDRESS 0x12		/* address of rock DLL string */
#define PP_OLD_ENTRY_ADDRESS 0x2B		/* address of old entry code buffer */

BOOL insert_code(DWORD entry_address, DWORD preload_address,
                 PBYTE old_entry_code, HANDLE process);

#endif