/*
 * pi_unix.c
 *
 * Platform-specific routines for unix.
 */

#include "pi.h"
#include "rs_unix.h"

#include <errno.h>
#include <stdio.h>
#include <stdarg.h>

#include <fcntl.h>
#include <sys/ioctl.h>

int pi_closesocket(int sd) {

	return close(sd);
}

int pi_closefile(int fd) {

	return close(fd);
}

int pi_ioctl(int d, int req, ioctlarg_t arg) {

	return ioctl(d, req, arg);
}

int pi_gettimeofday(struct timeval *tv, struct timezone *tz) {

	return gettimeofday(tv, tz);
}

int pi_getpid() {

	return getpid();
}

int pi_dup(int fd) {

	return dup(fd);
}

int pi_dup2(int fd1, int fd2) {

	return dup2(fd1, fd2);
}

int pi_snprintf(char *buf, size_t len, const char *format, ...) {

	va_list ap;
	int n;

	va_start(ap, format);
	n = vsnprintf(buf, len, format, ap);
	va_end(ap);

	return n;
}

int pi_isatty(int fd) {

	return isatty(fd);
}

int pi_nonblock(int sd, int onoff) {

	int flags;

        flags = fcntl(sd, F_GETFL, 0);
        if (0 > flags)
                return -1;
        if (onoff)
                flags |= O_NONBLOCK;
        else
                flags &= ~O_NONBLOCK;

	return fcntl(sd, F_SETFL, flags);
}

int pi_socket_errno() {

	return errno;
}

void pi_set_socket_errno(int e) {

	errno = e;
}

int pi_sitfor(struct timeval *tv) {

	return select(0, NULL, NULL, NULL, tv);
}

int rs_platform_init() { return 0; }
int rs_platform_socket(rs_t rs) { return 0; }
int rs_platform_connect(rs_t rs) { return 0; }
int rs_platform_free(rs_t rs) { return 0; }
