/*
 *  rocks/pi_win.c
 *
 *  PI impementation for windows.
 *
 *  Copyright (C) 2001 Victor Zandy, Greg Quinn
 *  See COPYING for distribution terms.
 */

#include "pi.h"
#include "rs_win.h"

#include <stdio.h>
#include <stdarg.h>
#include <io.h>
#include <assert.h>

int pi_closesocket(SOCKET sd) {

	return closesocket(sd);
}

int pi_closefile(int handle) {

	return _close(handle);
}

int pi_ioctl(SOCKET s, long cmd, unsigned long *arg) {

	return ioctlsocket(s, cmd, arg);
}

int pi_gettimeofday(struct timeval *tv, timezone_t null) {

	DWORD t = GetTickCount();
	tv->tv_sec = t / 1000;
	tv->tv_usec = t % 1000;

	return 0;
}

int pi_getpid() {

	return GetCurrentProcessId();
}

int pi_dup(int handle) {

	return _dup(handle);
}

int pi_dup2(int handle1, int handle2) {

	return _dup2(handle1, handle2);
}

int pi_snprintf(char *buf, size_t len, const char *format, ...) {

	va_list ap;
	int n;

	va_start(ap, format);
	n = _vsnprintf(buf, len, format, ap);
	va_end(ap);

	return n;
}

int pi_isatty(int handle) {

	return _isatty(handle);
}

int pi_nonblock(SOCKET sd, int onoff) {

	unsigned long ul = (unsigned long)onoff;

	if (!onoff)
		assert(WSAEventSelect(sd, NULL, 0) == 0);
	if (ioctlsocket(sd, FIONBIO, &onoff) == SOCKET_ERROR)
		return -1;

	return 0;
}

int pi_socket_errno() {

	return WSAGetLastError();
}

void pi_set_socket_errno(int e) {

	WSASetLastError(e);
}

int pi_sitfor(struct timeval *tv) {

	Sleep(tv->tv_sec * 1000 + tv->tv_usec);
	return 0;
}

int rs_platform_init() {

	/* begin network event handling thread */
	init_net_events();

	return 0;
}

int rs_platform_socket(rs_t rs) {

	/* make event objects for control thread synchronization */
	rs->plat_dep.con_event = CreateEvent(NULL, FALSE, FALSE, NULL);
	if (rs->plat_dep.con_event == NULL)
		goto error;
	rs->plat_dep.fail_event = CreateEvent(NULL, TRUE, FALSE, NULL);
	if (rs->plat_dep.fail_event == NULL)
		goto error;
	rs->plat_dep.close_event = CreateEvent(NULL, FALSE, FALSE, NULL);
	if (rs->plat_dep.close_event == NULL)
		goto error;

	/* reconnection event object for synchronization on blocking calls */
	rs->plat_dep.rec_event = CreateEvent(NULL, TRUE, TRUE, NULL);
	if (rs->plat_dep.rec_event == NULL)
		goto error;

	/* begin execution of the control thread */
	rs_start_ctl(rs);

	return 0;

error:
	if (rs->plat_dep.rec_event != NULL)
		CloseHandle(rs->plat_dep.rec_event);
	if (rs->plat_dep.fail_event != NULL)
		CloseHandle(rs->plat_dep.fail_event);
	if (rs->plat_dep.con_event != NULL)
		CloseHandle(rs->plat_dep.con_event);
	if (rs->plat_dep.close_event != NULL)
		CloseHandle(rs->plat_dep.close_event);
	return -1;
}

int rs_platform_connect(rs_t rs) {

	event_rebind(rs);

	assert(SetEvent(rs->plat_dep.con_event));

	return 0;
}

int rs_platform_free(rs_t rs) {

	assert(SetEvent(rs->plat_dep.close_event));
	WaitForSingleObject(rs->plat_dep.ctl_thread, INFINITE);
	event_unrock(rs);
	CloseHandle(rs->plat_dep.rec_event);
	CloseHandle(rs->plat_dep.ctl_thread);
			
	return 0;
}

void rs_fake_disconnect(socket_t sd) {

	rs_t rs = rs_lookup(sd);
	rs_reset_on_close(rs->sd, 1);
	pi_closesocket(rs->sd);
}
