/*
 * rocks/pi_win.h
 *
 * Type abstractions and constants to hide windows
 * platform specifics
 */

#ifndef _PI_WIN_H
#define _PI_WIN_H

#include <winsock2.h>

/* platform dependent rs structure */
typedef struct {
	SOCKET name;			/* name of socket to application */
	HANDLE con_event;		/* for signalling connection to ctl thread */
	HANDLE fail_event;		/* for signalling failure to ctl thread */
	HANDLE close_event;		/* for signalling close to control thread */
	HANDLE rec_event;		/* for blocking calls waiting on reconnection */
	HANDLE ctl_thread;		/* control thread */
	DWORD ctl_thread_id;	/* control thread id */
	SOCKET rec_notify_sd;	/* reconnection notification socket */
	SOCKET rec_select_sd;	/* select socket for suspended rocks */
	struct sockaddr_in rec_notify_addr;
} rs_platform_t;

/* type for socket descriptors */
typedef SOCKET socket_t;

/* type for length of socket address structures */
typedef int salen_t;

/* type for send, recv buffers */
typedef char * iobuff_t;

/* type for length of io buffers */
typedef int iosize_t;

/* type for for return value from recv, send */
typedef int ioret_t;

/* types for ioctl's */
typedef long ioctlcmd_t;
typedef unsigned long * ioctlarg_t; 

/* dummy timezone type for emulated gettimeofday */
typedef void * timezone_t;

/* pid type */
typedef int pid_t;

/* for now, we are just giving UNIX names to windows bitflags */
#define SOL_TCP IPPROTO_TCP
#define EPROTONOSUPPORT WSAEPROTONOSUPPORT
#define EISCONN WSAEISCONN
#define ECONNRESET WSAECONNRESET
#define EINPROGRESS WSAEINPROGRESS
#define EWOULDBLOCK WSAEWOULDBLOCK
#define EALREADY WSAEALREADY

/* default log file */
#define DEFAULT_LOG "c:/temp/rocks.log"

#endif
