/* 
 *  rocks/rock.c
 *
 *  Reliable sockets program launcher. Inject the rocks DLL into
 *  the desired process and let it run.
 *
 *  Copyright (C) 2001 Victor Zandy, Greg Quinn
 *  See COPYING for distribution terms.
 */

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "insert_code.h"

#define MAX_EXE_NAME 1000

int get_exe_name(PSTR cmd, PSTR exe, size_t size) {

	size_t n = strlen(cmd);
	if (n > size - 4)
		return -1;
	memcpy(exe, cmd, n);
	if (n > 4)
		if (stricmp(".exe", &cmd[n - 4])) {
			memcpy(&exe[n], ".exe", 4);
			n += 4;
		}
	exe[n] = '\0';

	return 0;
}

BOOL get_entry_pt(PSTR exe, DWORD *entry_pt) {

	HANDLE file_handle = INVALID_HANDLE_VALUE;
	HANDLE map_handle = NULL;
	PVOID file_base = NULL;
	PIMAGE_DOS_HEADER dos_header;
	PIMAGE_NT_HEADERS nt_header;

	/* set up memory-mapped file */
	file_handle = CreateFile(exe, GENERIC_READ, FILE_SHARE_READ,
	                         NULL, OPEN_EXISTING, 0, NULL);
	if (file_handle == INVALID_HANDLE_VALUE)
		goto error;
	map_handle = CreateFileMapping(file_handle, NULL, PAGE_READONLY, 0, 0, NULL);
	if (map_handle == NULL)
		goto error;
	file_base = MapViewOfFile(map_handle, FILE_MAP_READ, 0, 0, 0);
	if (file_base == NULL)
		goto error;

	/* retrieve the entry-point address */
	dos_header = (PIMAGE_DOS_HEADER)file_base;
	if (dos_header->e_magic != IMAGE_DOS_SIGNATURE)
		goto error;
	nt_header = (PIMAGE_NT_HEADERS)((char *)file_base + dos_header->e_lfanew);
	if (nt_header->Signature != IMAGE_NT_SIGNATURE)
		goto error;
	if (nt_header->FileHeader.Machine != IMAGE_FILE_MACHINE_I386)
		goto error;
	if (nt_header->OptionalHeader.Magic != IMAGE_NT_OPTIONAL_HDR32_MAGIC)
		goto error;
	*entry_pt = nt_header->OptionalHeader.ImageBase +
		nt_header->OptionalHeader.AddressOfEntryPoint;

	return TRUE;

error:
	if (file_base != NULL)
		UnmapViewOfFile(file_base);
	if (map_handle != NULL)
		CloseHandle(map_handle);
	if (file_handle != INVALID_HANDLE_VALUE)
		CloseHandle(file_handle);
	return FALSE;
}

int WINAPI WinMain(HINSTANCE inst, HINSTANCE prev_inst, PSTR cmd_line,
				   int cmd_show) {

	STARTUPINFO startup_info;
	PROCESS_INFORMATION process_info;
	BOOL success;
	CHAR exe_name[MAX_EXE_NAME];
	DWORD entry_address, preload_address;
	BYTE old_entry_code[ENTRY_BIN_SIZE];
	int exit_code = 0;
	DWORD scratch;

	if (__argc < 2) {
		MessageBox(NULL, TEXT("Usage: Rock_win.exe <command>"),
			       TEXT("ROCKWIN.EXE"), MB_OK);
		return -1;
	}

	/* start the child suspended */
	memset(&startup_info, 0, sizeof(startup_info));
	startup_info.cb = sizeof(startup_info);
	success = CreateProcess(NULL, cmd_line, NULL, NULL, FALSE,
	                        CREATE_SUSPENDED, NULL, NULL, &startup_info,
							&process_info);
	if (!success) {
		MessageBox(NULL, TEXT("Unable to create the child process."),
		           TEXT("Rocks Library Initialization"), MB_OK);
		return -1;
	}

	/* get the address of the entry point */
	if (get_exe_name(__argv[1], exe_name, sizeof(exe_name))) {
		MessageBox(NULL, TEXT("GetModuleFileName error"),
		           TEXT("Rocks Library Initialization"), MB_OK);
		goto kill;
	}
	success = get_entry_pt(exe_name, &entry_address);
	if (!success) {
		MessageBox(NULL, TEXT("Could not get entry point of executable."),
		           TEXT("Rocks Library Initialization"), MB_OK);
		goto kill;
	}

	/* save the old entry-point code */
	ReadProcessMemory(process_info.hProcess, (PCHAR)NULL + entry_address,
	                  old_entry_code, ENTRY_BIN_SIZE, &scratch);
	if (scratch != ENTRY_BIN_SIZE) {
		MessageBox(NULL, TEXT("Error reading child process memory"),
		           TEXT("Rocks Library Initialization"), MB_OK);
		goto kill;
	}

	/* allocate memory for preload code in child process */
	preload_address = (DWORD)((PCHAR)VirtualAllocEx(process_info.hProcess, NULL,
	                                                PRELOAD_BIN_SIZE, MEM_COMMIT,
									                PAGE_EXECUTE_READWRITE) -
	                          (PCHAR)NULL);
	if (preload_address == 0) {
		MessageBox(NULL, TEXT("Error allocating memory in child."),
		           TEXT("Rocks Library Initialization"), MB_OK);
		goto kill;
	}

	/* insert the replacement code */
	if (!insert_code(entry_address, preload_address, old_entry_code,
	                 process_info.hProcess)) {
		goto kill;
	}

	if (ResumeThread(process_info.hThread) == -1) {
		MessageBox(NULL, TEXT("Error resuming child thread."),
		           TEXT("Rocks Library Initialization"), MB_OK);
		goto kill;
	}

	return 0;

kill:
	TerminateProcess(process_info.hProcess, -1);
	return -1;
}