/*
*  rocks/rs_unix.h
*
*  UNIX-specifics for rocks.
*
*  Copyright (C) 2003 Victor Zandy, Greg Quinn
*  See COPYING for distribution terms.
*/

#ifndef _RS_UNIX_H
#define _RS_UNIX_H

#include "rs.h"

int rs_fork();
int rs_vfork();
unsigned int rs_alarm(unsigned int);
int rs_read(int fd, void *buf, size_t len);
int rs___read(int fd, void *buf, size_t len);
int rs_write(int fd, const void *buf, size_t len);
int rs_fdset(fd_set *);
ssize_t rs_readv(int sd, const struct iovec *iov, int iovcnt);
ssize_t rs_writev(int sd, const struct iovec *iov, int iovcnt);
int rs_recvmsg(int sd, struct msghdr *msg, int flags);
int rs_sendmsg(int sd, const struct msghdr *msg, int flags);
int rs_dup(int old);
int rs_dup2(int old, int new);

/* Overloaded syscalls needed for our sanity, not for the sockets API */
typedef void (*sighandler_t)(int);
int rs_sigaction(int sig, const struct sigaction *act, struct sigaction *oldact);
int rs___libc_sigaction(int sig, const struct sigaction *act, struct sigaction *oldact);
sighandler_t rs_signal(int signum, sighandler_t handler);
int rs_sigprocmask(int how, const sigset_t *set, sigset_t *oldset);
int rs_sigsuspend(const sigset_t *mask);
int rs_sigaltstack(const stack_t *ss, stack_t *oss);

/* util.c stuff */
void rs_kill9_and_wait(pid_t pid);
int rs_rocklist_insert(rocklist_t *head, rs_t rs);
int rs_rocklist_remove(rocklist_t *head, rs_t rs);
rs_t rs_rocklist_findsa(rocklist_t head, struct sockaddr_in *sa);
void rs_rocklist_free(rocklist_t *head);
void rs_rs_sigaction(int sig, const struct sigaction *sa);
int rs_fcntl(int fd, int cmd, long arg);
int rs_setitimer(int which, const struct itimerval *value,
                 struct itimerval *ovalue);
int rs_save(rs_t rs, int fd);
rs_t rs_restore(int fd);

int replace_function(char *from, void *to);

/* exec.c */
int rs_execve(const char *filename, char *const argv[], char *const envp[]);

/* hb.c */
void rs_stop_heartbeat(sigset_t *);
void rs_resume_heartbeat(sigset_t *);
int rs_hb_save(hb_t hb, int fd);
hb_t rs_hb_restore(rs_t rs, int fd);

/* rs_unix.h */
void rs_share_rockfd(int, int);
char *rs_getargv0();


#endif
