/* 
 *  rocks/rslist_win.c
 *
 *  Mapping layer for SOCKET namespace in windows.
 *
 *  Copyright (C) 2001 Victor Zandy, Greg Quinn
 *  See COPYING for distribution terms.
 */

#include "rs_win.h"
#include "rslist.h"
#include <errno.h>
#include <assert.h>

/* list implementation for rocks structures */
struct rs_node {
	int has_rs;	        /* flag set if rs struct is present */
	union {
		rs_t rs;	/* rs struct which contains sd */
		SOCKET sd;	/* socket value used for winsock calls */
	} mapping;
	SOCKET name;		/* socket value returned to application */
	struct rs_node *next;
};
static struct rs_node *rocks;
static SOCKET next_sock = 64;

SOCKET rs_add_rock(rs_t rs) {

	struct rs_node *node;

	/* add to the rocks list */
	node = (struct rs_node *)malloc(sizeof(struct rs_node));
	if (node == NULL) {
		rserrno = ENOMEM;
		return INVALID_SOCKET;
	}
	node->name = next_sock++;
	node->has_rs = 1;
	node->mapping.rs = rs;
	node->next = rocks;
	rs->plat_dep.name = node->name;
	rocks = node;

	return node->name;
}

SOCKET rs_add_nonrock(SOCKET sd) {

	struct rs_node *node;

	/* add to the rocks list */
	node = (struct rs_node *)malloc(sizeof(struct rs_node));
	if (node == NULL) {
		rserrno = ENOMEM;
		return INVALID_SOCKET;
	}
	node->name = next_sock++;
	node->has_rs = 0;
	node->mapping.sd = sd;
	node->next = rocks;
	rocks = node;

	return node->name;
}

/* removes node for given socket name (should be only one) */
int rs_remove_node(SOCKET name) {

	struct rs_node *node, **last;

	node = rocks;
	last = &rocks;
	while (node != NULL) {
		if (node->name == name) {
			*last = node->next;
			free(node);
			return 1;
		}
		last = &node->next;
		node = node->next;
	}
	return 0;
}

#if 0
/* remove all nodes pointing to a particular rock */
int remove_rs_nodes(rs_t rs) {

	struct rs_node *node, **last;
	int removed = 0;

	node = rocks;
	last = &rocks;
	while (node != NULL) {
		if (node->has_rs && node->mapping.rs == rs) {
			*last = node->next;
			free(node);
			node = *last;
			removed++;
		}
		else {
			last = &node->next;
			node = node->next;
		}
	}
	return removed;
}
#endif

SOCKET rs_lookup_socket(SOCKET name) {

	struct rs_node *node = rocks;

	while (node != NULL) {
		if (node->name == name)
			if (node->has_rs)
				return node->mapping.rs->sd;
			else
				return node->mapping.sd;
		node = node->next;
	}

	return INVALID_SOCKET;
}

SOCKET rs_lookup_reverse(SOCKET sd) {

	struct rs_node *node = rocks;

	while (node != NULL) {
		if (node->has_rs) {
			if (node->mapping.rs->sd == sd)
				return node->name;
		}
		else if (node->mapping.sd == sd)
			return node->name;

		node = node->next;
	}

	return INVALID_SOCKET;
}

rs_t rs_lookup(SOCKET name) {

	struct rs_node *node = rocks;

	while (node != NULL) {
		if (node->name == name)
			if (node->has_rs)
				return node->mapping.rs;
			else
				return NULL;
		node = node->next;
	}

	return NULL;
}

void rs_unrock(rs_t rs) {

	struct rs_node *node = rocks;
	
	while (node != NULL) {
		if (node->has_rs && node->mapping.rs == rs) {
			node->has_rs = 0;
			node->mapping.sd = rs->sd;
		}
		node = node->next;
	}
}

SOCKET rs_rebind_rock(rs_t rs, SOCKET sd) {

	rs->sd = sd;
	event_rebind(rs);
	return sd;
}
