/*
 *  rocks/select_win.c
 *
 *  Copyright (C) 2003 Victor Zandy, Greg Quinn
 *  See COPYING for distribution terms.
 */

#include "rs_win.h"
#include "rslist.h"
#include <assert.h>

static void choose(fd_set *src, fd_set *dst, fd_set *sdst, fd_set *susp) {

	unsigned i;

	for (i = 0; i < src->fd_count; i++) {
		rs_t rock = rs_lookup(src->fd_array[i]);
		if (rock && rock->state == RS_SUSPENDED) {
			if (!FD_ISSET(rock->plat_dep.rec_select_sd, susp)) {
				WSAPROTOCOL_INFO info;
				SOCKET sd;

				assert(WSADuplicateSocket(rock->plat_dep.rec_select_sd,
					GetCurrentProcessId(), &info) == 0);
				assert((sd = WSASocket(FROM_PROTOCOL_INFO,
				                       FROM_PROTOCOL_INFO,
									   FROM_PROTOCOL_INFO,
									   &info, 0, 0)) != INVALID_SOCKET);
				FD_SET(sd, sdst);
				FD_SET(sd, susp);
			}
		}
		else
			FD_SET(rs_lookup_socket(src->fd_array[i]), dst);
	}
}

static int
check_spilled(fd_set *rset, fd_set *spilled)
{
	unsigned i;
	int nspilled = 0;

	for (i = 0; i < rset->fd_count; i++) {
		rs_t x = rs_lookup(rset->fd_array[i]);
		if (x && (x->edpspill || x->clospill)) {
				nspilled++;
				FD_SET(x->sd, spilled);
		}
	}

	return nspilled;
}

static void translate(fd_set *dst, fd_set *src) {

	unsigned i;

	for (i = 0; i < src->fd_count; i++) {
		FD_SET(rs_lookup_reverse(src->fd_array[i]), dst);
		assert(dst->fd_array[i] != INVALID_SOCKET);
	}
}

int rs_select(int n, fd_set *rs, fd_set *ws, fd_set *es, struct timeval *tv) {

	int rv;
	fd_set r, w, e;
	fd_set suspended;
	fd_set *rp, *wp, *ep;
	fd_set spilled;
	int nspilled = 0;
	struct timeval tv_spill = {0, 0};
	unsigned i;

	nspilled = 0;
	FD_ZERO(&spilled);
	if (rs)
		if (nspilled = check_spilled(rs, &spilled))
			tv = &tv_spill;

retry:
	FD_ZERO(&r);
	FD_ZERO(&w);
	FD_ZERO(&e);
	FD_ZERO(&suspended);
	if (rs)
		choose(rs, &r, &r, &suspended);
	if (ws)
		choose(ws, &w, &r, &suspended);
	if (es)
		choose(es, &e, &r, &suspended);

	if (r.fd_count) rp = &r; else rp = NULL;
	if (w.fd_count) wp = &w; else wp = NULL;
	if (e.fd_count) ep = &e; else ep = NULL;
	rv = select(n, rp, wp, ep, tv);

	/* check for recovered rocks */
	if (rp) {
		int recovered = 0;
		for (i = 0; i < suspended.fd_count; i++) {
			if (FD_ISSET(suspended.fd_array[i], rp))
				recovered++;
		}
		if (recovered) goto retry;
	}

	/* merge in spilled rocks */
	if (nspilled) {
		if (!rp) {
			rp = &r;
			FD_ZERO(rp);
		}
		for (i = 0; i < spilled.fd_count; i++)
			FD_SET(spilled.fd_array[i], rp);
	}

	/* translate fd_sets for caller */
	if (rp && rs) {
		FD_ZERO(rs);
		translate(rs, rp);
	}
	if (wp) {
		FD_ZERO(ws);
		translate(ws, wp);
	}
	if (ep) {
		FD_ZERO(es);
		translate(es, ep);
	}

	/* close dup'ed sockets */
	for (i = 0; i < suspended.fd_count; i++)
		closesocket(suspended.fd_array[i]);

	return rv;
}