/* 
 *  rocks/sockopt.c
 *
 *  Sockets API implementation for setsockopt and getsockopt.
 *
 *  Copyright (C) 2001 Victor Zandy
 *  See COPYING for distribution terms.
 */

#include "rs.h"
#include "log.h"

int
rs_setsockopt(socket_t s, int level, int optname,
              iobuff_t optval, salen_t optlen)
{
	if (level == SOL_SOCKET && optname == SO_KEEPALIVE) {
		rs_log("Ignoring app KEEPALIVE");
		return 0;
	}
	if (level == SOL_SOCKET && optname == SO_SNDBUF) {
		rs_log("Ignoring app SO_SNDBUF (%d)", *(unsigned long*)optval);
		return 0;
	}
	if (level == SOL_SOCKET && optname == SO_RCVBUF) {
		rs_log("Ignoring app SO_RCVBUF (%d)", *(unsigned long*)optval);
		return 0;
	}
	if (level == SOL_SOCKET && optname == SO_LINGER) {
		struct linger *lo = (struct linger *)optval;
		rs_log("Ignoring app LINGER { l_onoff = %d, l_linger = %d }",
		       lo->l_onoff, lo->l_linger);
		return 0;
	}
	return setsockopt(s, level, optname, optval, optlen);
}

int
rs_ioctl(socket_t fd, ioctlcmd_t cmd, ioctlarg_t arg)
{
	return pi_ioctl(fd, cmd, arg);
}
