#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/ip.h>
#include <errno.h>
#include <sys/time.h>
#include <assert.h>
#include <signal.h>

#include "rocks.h"
#include "util.h"

static char SRVMSG[] = "does the server speak first?";
static unsigned SRVMSGLEN = sizeof(SRVMSG)-1;
static char CLTMSG[] = "or does the client?!?";
static unsigned CLTMSGLEN = sizeof(CLTMSG)-1;

enum {
	ROUNDS = 100,
	PERIOD = 10,
};

static void
srv(int fd)
{
	int c;
	char buf[CLTMSGLEN];
	int i, rv;

retry:
	c = accept(fd, NULL, NULL);
	if (0 > c)
		fatal("accept");

	for (i = 0; i < ROUNDS; i++) {
		rv = write(c, SRVMSG, SRVMSGLEN);
		if (0 > rv)
			fatal("write");
		if (rv != SRVMSGLEN)
			fatal("short write");
		rv = xread(c, buf, CLTMSGLEN);
		if (0 > rv)
			fatal("read");
		if (0 == rv) {
			/* hangup: could be client probe */
			close(c);
			goto retry;
		}
		if (rv != CLTMSGLEN)
			fatal("short read");
		if (strncmp(buf, CLTMSG, CLTMSGLEN))
			fatal("corrupt message");
	}
	close(c);
}

static void
clt(int c, unsigned short port)
{
	struct sockaddr_in addr;
	int rv;
	char buf[SRVMSGLEN];
	int i;

	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = inet_addr("127.0.0.1");	
	addr.sin_port = htons(port);
	if (0 > connect(c, (struct sockaddr *)&addr, sizeof(addr)))
		fatal("connect");

	for (i = 0; i < ROUNDS; i++) {
		rv = write(c, CLTMSG, CLTMSGLEN);
		if (0 > rv)
			fatal("write");
		if (rv != CLTMSGLEN)
			fatal("short write");

#if 1
		if (i%PERIOD == 0)
			rock_suspend(c);
#endif

		rv = xread(c, buf, SRVMSGLEN);
		if (0 > rv)
			fatal("read");
		if (0 == rv)
			fatal("unexpected eof");
		if (rv != SRVMSGLEN)
			fatal("short read");
		if (strncmp(buf, SRVMSG, SRVMSGLEN))
			fatal("corrupt message");
	}
}

int
main(int argc, char *argv[])
{
	unsigned short srvport;
	int s;
	int pid;

	s = socket(AF_INET, SOCK_STREAM, 0);
	if (0 > s)
		fatal("socket");
	pid = fork_srv(s, srv, &srvport);
	close(s);

	s = socket(AF_INET, SOCK_STREAM, 0);
	if (0 > s)
		fatal("socket");
	clt(s, srvport);
	close(s);
	kill(pid, SIGTERM);
	return 0;
}
