#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/ip.h>
#include <errno.h>
#include <sys/time.h>
#include <assert.h>
#include <signal.h>

void
fatal(char *how)
{
	if (errno)
		fprintf(stderr, "%s: %s\n", how, strerror(errno));
	else
		fprintf(stderr, "%s\n", how);
	kill(-getpgrp(), SIGTERM);
	assert(0);
}

static void
reuseaddr(int sd)
{
	int optval = 1;
	if (0 > setsockopt(sd, SOL_SOCKET, SO_REUSEADDR,
			   &optval, sizeof(optval)))
		fatal("SO_REUSEADDR");
}

int
xread(int sd, void *buf, size_t len)
{
	char *p = (char *)buf;
	size_t nrecv = 0;
	ssize_t rv;

	assert(len > 0); 	/* a common bug */
	while (nrecv < len) {
		rv = read(sd, p, len - nrecv);
		if (0 > rv && errno == EINTR)
			continue;
		if (0 > rv)
			return -1;
		if (0 == rv)
			return 0;
		nrecv += rv;
		p += rv;
	}
	return nrecv;
}

/* bind S to a listening server on an arbitrary port
   and call SRV on S in a new process;
   set PORT to the port number in host order
   and return the pid of the new process */
int
fork_srv(int s, void(*srv)(int s), unsigned short *port)
{
	int pid;
	socklen_t len;
	struct sockaddr_in saddr;

	bzero(&saddr, sizeof(saddr));
	saddr.sin_family = AF_INET;
	saddr.sin_addr.s_addr = INADDR_ANY;
	saddr.sin_port = htons(0);
	reuseaddr(s);
	if (0 > bind(s, (struct sockaddr *)&saddr, sizeof(saddr)))
		fatal("bind");
	if (0 > listen(s, 1))
		fatal("listen");
	len = sizeof(saddr);
	if (0 > getsockname(s, (struct sockaddr *)&saddr, &len))
		fatal("getsockname");
	pid = fork();
	if (0 > pid)
		fatal("fork");
	else if (pid) {
		*port = ntohs(saddr.sin_port); 
		return pid;
	} else {
		srv(s);
		exit(0);
	}
	return -1;
}


void
result(char *test, char *res)
{
	char buf[60];
	int len;
		
	memset(buf, '.', sizeof(buf));
	buf[sizeof(buf)-1] = '\0';
	len = strlen(test);
	if (len > sizeof(buf)-5)
		len = sizeof(buf)-5;
	strncpy(buf, test, len);
	printf("%s%s\n", buf, res);
}
